@@ -152,7 +152,9 @@ SlowHTTPTest::SlowHTTPTest(int delay, int duration,
 }
 
 SlowHTTPTest::~SlowHTTPTest() {
-  freeaddrinfo(addr_);
+  if (addr_) {
+    freeaddrinfo(addr_);
+  }
 
   for(std::vector<StatsDumper*>::iterator i = dumpers_.begin();
        i != dumpers_.end(); ++i) {
@@ -213,7 +215,7 @@ const char* SlowHTTPTest::get_random_extra() {
 
 bool SlowHTTPTest::init(const char* url, const char* verb,
     const char* path, const char* proxy,
-    const char* content_type, const char* accept) {
+    const char* content_type, const char* accept, const char* cookie) {
   if(!change_fd_limits()) {
     slowlog(LOG_INFO, "error setting open file limits\n");
     
@@ -331,6 +333,13 @@ bool SlowHTTPTest::init(const char* url, const char* verb,
   request_.append(user_agent_);
   request_.append("\r\n");
   request_.append(referer);
+  // Cookie
+  if (cookie != 0 && strlen(cookie)) {
+    cookie_.append(cookie);
+    request_.append("Cookie: ");
+    request_.append(cookie_);
+    request_.append(crlf);
+  }
   // method for probe is always GET
   probe_request_.append("GET");
   if(eProbeProxy == proxy_type_) {
@@ -390,6 +399,7 @@ bool SlowHTTPTest::init(const char* url, const char* verb,
           "<tr><td><b>Number of connections</b></td><td>%d</td></tr>"
           "<tr><td><b>Verb</b></td><td>%s</td></tr>"
           "<tr><td><b>Content-Length header value</b></td><td>%d</td></tr>"
+          "<tr><td><b>Cookie</b></td><td>%s</td></tr>"
           "<tr><td><b>Extra data max length</b></td><td>%d</td></tr>"
           "<tr><td><b>Interval between follow up data</b></td><td>%d seconds</td></tr>"
           "<tr><td><b>Connections per seconds</b></td><td>%d</td></tr>"
@@ -401,6 +411,7 @@ bool SlowHTTPTest::init(const char* url, const char* verb,
           num_connections_,
           verb_.c_str(),
           content_length_,
+          cookie,
           extra_data_max_len_total_,
           followup_timing_,
           delay_,
@@ -414,6 +425,7 @@ bool SlowHTTPTest::init(const char* url, const char* verb,
           "<tr><th>Test parameters</th></tr>"
           "<tr><td><b>Test type</b></td><td>%s</td></tr>"
           "<tr><td><b>Number of connections</b></td><td>%d</td></tr>"
+          "<tr><td><b>Cookie</b></td><td>%s</td></tr>"
           "<tr><td><b>Receive window range</b></td><td>%d - %d</td></tr>"
           "<tr><td><b>Pipeline factor</b></td><td>%d</td></tr>"
           "<tr><td><b>Read rate from receive buffer</b></td><td>%d bytes / %d sec</td></tr>"
@@ -424,6 +436,7 @@ bool SlowHTTPTest::init(const char* url, const char* verb,
           "</table>",
           test_type_name[test_type_],
           num_connections_,
+          cookie,
           window_lower_limit_,
           window_upper_limit_,
           pipeline_factor_,
@@ -483,6 +496,7 @@ void SlowHTTPTest::report_parameters() {
       cBLU "number of connections:" cLBL "            %d\n"
       cBLU "URL:" cLBL "                              %s\n"
       cBLU "verb:" cLBL "                             %s\n"
+      cBLU "cookie:" cLBL "                           %s\n"
       cBLU "Content-Length header value:" cLBL "      %d\n"
       cBLU "follow up data max size:" cLBL "          %d\n"
       cBLU "interval between follow up data:" cLBL "  %d seconds\n"
@@ -494,6 +508,7 @@ void SlowHTTPTest::report_parameters() {
         num_connections_,
         base_uri_.getData(),
         verb_.c_str(),
+        cookie_.c_str(),
         content_length_,
         extra_data_max_len_total_,
         followup_timing_,
@@ -510,6 +525,7 @@ void SlowHTTPTest::report_parameters() {
       cBLU "number of connections:" cLBL "           %d\n"
       cBLU "URL:" cLBL "                             %s\n"
       cBLU "verb:" cLBL "                            %s\n"
+      cBLU "cookie:" cLBL"                           %s\n"
       cBLU "receive window range:" cLBL "            %d - %d\n"
       cBLU "pipeline factor:" cLBL "                 %d\n"
       cBLU "read rate from receive buffer:" cLBL "   %d bytes / %d sec\n"
@@ -521,6 +537,7 @@ void SlowHTTPTest::report_parameters() {
         num_connections_,
         base_uri_.getData(),
         verb_.c_str(),
+        cookie_.c_str(),
         window_lower_limit_,
         window_upper_limit_,
         pipeline_factor_,
